@@ -1,6 +1,6 @@
 /*
  * ModSecurity for Apache 2.x, http://www.modsecurity.org/
- * Copyright (c) 2004-2008 Breach Security, Inc. (http://www.breach.com/)
+ * Copyright (c) 2004-2009 Breach Security, Inc. (http://www.breach.com/)
  *
  * This product is released under the terms of the General Public Licence,
  * version 2 (GPLv2). Please refer to the file LICENSE (included with this
@@ -25,15 +25,6 @@
 #include "msc_util.h"
 #include "msc_xml.h"
 
-modsec_build_type_rec DSOLOCAL modsec_build_type[] = {
-    { "-dev", 1 },     /* Development build */
-    { "-rc", 3 },      /* Release Candidate build */
-    { "", 9 },         /* Production build */
-    { "-breach", 9 },  /* Breach build */
-    { "-trunk", 9 },   /* Trunk build */
-    { NULL, -1 }       /* terminator */
-};
-
 /**
  * Log an alert message to the log, adding the rule metadata at the end.
  */
@@ -268,6 +259,12 @@ apr_status_t modsecurity_tx_init(modsec_rec *msr) {
         }
     }
 
+    /* Check if we are forcing buffering, then use memory only. */
+    if (msr->txcfg->reqbody_buffering) {
+        msr->msc_reqbody_storage = MSC_REQBODY_MEMORY;
+        msr->msc_reqbody_spilltodisk = 0;
+    }
+
     /* Initialise arguments */
     msr->arguments = apr_table_make(msr->mp, 32);
     if (msr->arguments == NULL) return -1;
@@ -314,6 +311,8 @@ apr_status_t modsecurity_tx_init(modsec_rec *msr) {
     msr->geo_vars = apr_table_make(msr->mp, 8);
     if (msr->geo_vars == NULL) return -1;
 
+    msr->collections_original = apr_table_make(msr->mp, 8);
+    if (msr->collections_original == NULL) return -1;
     msr->collections = apr_table_make(msr->mp, 8);
     if (msr->collections == NULL) return -1;
     msr->collections_dirty = apr_table_make(msr->mp, 8);
@@ -576,19 +575,14 @@ apr_status_t modsecurity_process_phase(modsec_rec *msr, unsigned int phase) {
     switch(phase) {
         case 1 :
             return modsecurity_process_phase_request_headers(msr);
-            break;
         case 2 :
             return modsecurity_process_phase_request_body(msr);
-            break;
         case 3 :
             return modsecurity_process_phase_response_headers(msr);
-            break;
         case 4 :
             return modsecurity_process_phase_response_body(msr);
-            break;
         case 5 :
             return modsecurity_process_phase_logging(msr);
-            break;
         default :
             msr_log(msr, 1, "Invalid processing phase: %d", msr->phase);
             break;
