@@ -1,6 +1,6 @@
 /*
  * ModSecurity for Apache 2.x, http://www.modsecurity.org/
- * Copyright (c) 2004-2008 Breach Security, Inc. (http://www.breach.com/)
+ * Copyright (c) 2004-2009 Breach Security, Inc. (http://www.breach.com/)
  *
  * This product is released under the terms of the General Public Licence,
  * version 2 (GPLv2). Please refer to the file LICENSE (included with this
@@ -32,23 +32,7 @@ typedef struct msc_data_chunk msc_data_chunk;
 typedef struct msc_arg msc_arg;
 typedef struct msc_string msc_string;
 
-#if !(defined(WIN32) || defined(CYGWIN) || defined(NETWARE) || defined(SOLARIS2))
-#define DSOLOCAL __attribute__((visibility("hidden")))
-#else
-#define DSOLOCAL
-#endif
-
-#if defined(DEBUG_MEM)
-/* Nothing Yet */
-#endif
-
-/* For GNU C, tell the compiler to check printf like formatters */
-#if (defined(__GNUC__) && !defined(SOLARIS2))
-#define PRINTF_ATTRIBUTE(a,b) __attribute__((format (printf, a, b)))
-#else
-#define PRINTF_ATTRIBUTE(a,b)
-#endif
-
+#include "msc_release.h"
 #include "msc_logging.h"
 #include "msc_multipart.h"
 #include "msc_pcre.h"
@@ -66,27 +50,6 @@ typedef struct msc_string msc_string;
 #include "http_log.h"
 #include "http_protocol.h"
 
-typedef struct modsec_build_type_rec {
-    const char * name;
-    int          val;
-} modsec_build_type_rec;
-extern DSOLOCAL modsec_build_type_rec modsec_build_type[];
-
-#define MODSEC_VERSION_MAJOR       "2"
-#define MODSEC_VERSION_MINOR       "5"
-#define MODSEC_VERSION_MAINT       "6"
-#define MODSEC_VERSION_TYPE        ""
-#define MODSEC_VERSION_RELEASE     ""
-
-#define MODULE_NAME "ModSecurity for Apache"
-
-#define MODSEC_VERSION_SUFFIX MODSEC_VERSION_TYPE MODSEC_VERSION_RELEASE
-#define MODULE_RELEASE \
-  MODSEC_VERSION_MAJOR "." MODSEC_VERSION_MINOR "." MODSEC_VERSION_MAINT \
-  MODSEC_VERSION_SUFFIX
-
-#define MODULE_NAME_FULL MODULE_NAME "/" MODULE_RELEASE " (http://www.modsecurity.org/)"
-
 #define PHASE_REQUEST_HEADERS       1
 #define PHASE_REQUEST_BODY          2
 #define PHASE_RESPONSE_HEADERS      3
@@ -363,6 +326,7 @@ struct modsec_rec {
     int                  upload_remove_files;
 
     /* other */
+    apr_table_t         *collections_original;
     apr_table_t         *collections;
     apr_table_t         *collections_dirty;
 
@@ -398,6 +362,7 @@ struct directory_config {
 
     int                  is_enabled;
     int                  reqbody_access;
+    int                  reqbody_buffering;
     long int             reqbody_inmemory_limit;
     long int             reqbody_limit;
     long int             reqbody_no_files_limit;
