@@ -1,6 +1,6 @@
 /*
  * ModSecurity for Apache 2.x, http://www.modsecurity.org/
- * Copyright (c) 2004-2008 Breach Security, Inc. (http://www.breach.com/)
+ * Copyright (c) 2004-2009 Breach Security, Inc. (http://www.breach.com/)
  *
  * This product is released under the terms of the General Public Licence,
  * version 2 (GPLv2). Please refer to the file LICENSE (included with this
@@ -343,14 +343,14 @@ static void sec_auditlog_write_producer_header(modsec_rec *msr) {
 
     /* Try to write everything in one go. */
     if (msr->txcfg->component_signatures->nelts == 0) {
-        text = apr_psprintf(msr->mp, "Producer: %s.\n", MODULE_NAME_FULL);
+        text = apr_psprintf(msr->mp, "Producer: %s.\n", MODSEC_MODULE_NAME_FULL);
         sec_auditlog_write(msr, text, strlen(text));
 
         return;
     }
 
     /* Start with the ModSecurity signature. */
-    text = apr_psprintf(msr->mp, "Producer: %s", MODULE_NAME_FULL);
+    text = apr_psprintf(msr->mp, "Producer: %s", MODSEC_MODULE_NAME_FULL);
     sec_auditlog_write(msr, text, strlen(text));
 
 
