@@ -1,6 +1,6 @@
 /*
  * ModSecurity for Apache 2.x, http://www.modsecurity.org/
- * Copyright (c) 2004-2008 Breach Security, Inc. (http://www.breach.com/)
+ * Copyright (c) 2004-2009 Breach Security, Inc. (http://www.breach.com/)
  *
  * This product is released under the terms of the General Public Licence,
  * version 2 (GPLv2). Please refer to the file LICENSE (included with this
@@ -33,6 +33,8 @@
 #define RESULT_WRONGSIZE        -3
 #define RESULT_WRONGRET         -4
 
+#define DEFAULT_ACTION           "phase:2,log,auditlog,pass"
+
 #define CMDLINE_OPTS             "t:n:p:P:r:I:D:Nh"
 
 /* Types */
@@ -126,6 +128,60 @@ void msr_log(modsec_rec *msr, int level, const char *text, ...) {
     va_end(ap);
 }
 
+void msr_log_error(modsec_rec *msr, const char *text, ...) {
+    va_list ap;
+    int level = 3;
+    char str1[1024] = "";
+    char str2[1256] = "";
+
+    if ((msr == NULL) || (level > msr->txcfg->debuglog_level)) {
+        return;
+    }
+    if (msr->txcfg->debuglog_fd == NOT_SET_P) {
+        if (apr_file_open(&msr->txcfg->debuglog_fd, msr->txcfg->debuglog_name, APR_READ|APR_WRITE|APR_CREATE|APR_APPEND|APR_BINARY, APR_OS_DEFAULT, g_mp) != APR_SUCCESS) {
+            fprintf(stderr, "ERROR: failed to create unit test debug log \"%s\".\n", msr->txcfg->debuglog_name);
+            msr->txcfg->debuglog_fd = NULL;
+        }
+    }
+
+    va_start(ap, text);
+    if (msr->txcfg->debuglog_fd != NULL) {
+        apr_size_t nbytes_written = 0;
+        apr_vsnprintf(str1, sizeof(str1), text, ap);
+        apr_snprintf(str2, sizeof(str2), "%lu: [%d] [%s] %s\n", (unsigned long)getpid(), level, test_name, str1);
+
+        apr_file_write_full(msr->txcfg->debuglog_fd, str2, strlen(str2), &nbytes_written);
+    }
+    va_end(ap);
+}
+
+void msr_log_warn(modsec_rec *msr, const char *text, ...) {
+    va_list ap;
+    int level = 4;
+    char str1[1024] = "";
+    char str2[1256] = "";
+
+    if ((msr == NULL) || (level > msr->txcfg->debuglog_level)) {
+        return;
+    }
+    if (msr->txcfg->debuglog_fd == NOT_SET_P) {
+        if (apr_file_open(&msr->txcfg->debuglog_fd, msr->txcfg->debuglog_name, APR_READ|APR_WRITE|APR_CREATE|APR_APPEND|APR_BINARY, APR_OS_DEFAULT, g_mp) != APR_SUCCESS) {
+            fprintf(stderr, "ERROR: failed to create unit test debug log \"%s\".\n", msr->txcfg->debuglog_name);
+            msr->txcfg->debuglog_fd = NULL;
+        }
+    }
+
+    va_start(ap, text);
+    if (msr->txcfg->debuglog_fd != NULL) {
+        apr_size_t nbytes_written = 0;
+        apr_vsnprintf(str1, sizeof(str1), text, ap);
+        apr_snprintf(str2, sizeof(str2), "%lu: [%d] [%s] %s\n", (unsigned long)getpid(), level, test_name, str1);
+
+        apr_file_write_full(msr->txcfg->debuglog_fd, str2, strlen(str2), &nbytes_written);
+    }
+    va_end(ap);
+}
+
 const char *ap_get_remote_host(conn_rec *conn, void *dir_config, int type, int *str_is_ip) {
     return "FAKE-REMOTE-HOST";
 }
@@ -268,7 +324,7 @@ static int init_op(op_data_t *data, const char *name, const char *param, unsigne
         *errmsg = apr_psprintf(g_mp, "Failed to create ruleset for op \"%s\".", name);
         return -1;
     }
-    data->rule = msre_rule_create(data->ruleset, RULE_TYPE_NORMAL, conf_fn, 1, "UNIT_TEST", args, "t:none,pass,nolog", errmsg);
+    data->rule = msre_rule_create(data->ruleset, RULE_TYPE_NORMAL, conf_fn, 1, "UNIT_TEST", args, DEFAULT_ACTION, errmsg);
     if (data->rule == NULL) {
         *errmsg = apr_psprintf(g_mp, "Failed to create rule for op \"%s\": %s", name, *errmsg);
         return -1;
@@ -402,6 +458,7 @@ static void init_msr() {
     dcfg = (directory_config *)apr_pcalloc(g_mp, sizeof(directory_config));
     dcfg->is_enabled = 0;
     dcfg->reqbody_access = 0;
+    dcfg->reqbody_buffering = 0;
     dcfg->reqbody_inmemory_limit = REQUEST_BODY_DEFAULT_INMEMORY_LIMIT;
     dcfg->reqbody_limit = REQUEST_BODY_DEFAULT_LIMIT;
     dcfg->reqbody_no_files_limit = REQUEST_BODY_NO_FILES_DEFAULT_LIMIT;
@@ -468,13 +525,16 @@ static void init_msr() {
     g_msr->hostname = "localhost";
     g_msr->msc_rule_mptmp = g_mp;
     g_msr->tx_vars = apr_table_make(g_mp, 1);
+    g_msr->collections_original = apr_table_make(g_mp, 1);
+    g_msr->collections = apr_table_make(g_mp, 1);
+    g_msr->collections_dirty = apr_table_make(g_mp, 1);
 }
 
 /**
  * Usage text.
  */
 static void usage() {
-    fprintf(stderr, "ModSecurity Unit Tester v%s\n", MODULE_RELEASE);
+    fprintf(stderr, "ModSecurity Unit Tester v%s\n", MODSEC_VERSION);
     fprintf(stderr, "  Usage: msc_test [options]\n");
     fprintf(stderr, "\n");
     fprintf(stderr, "  Options:\n");
@@ -759,6 +819,9 @@ int main(int argc, const char * const argv[])
         }
         else if (strcmp("action", type) == 0) {
             /* Actions */
+            int n;
+            const apr_array_header_t *arr;
+            apr_table_entry_t *te;
 
             rc = test_action(&action_data, &errmsg);
             if (rc < 0) {
@@ -780,6 +843,47 @@ int main(int argc, const char * const argv[])
                 ec = 1;
             }
 
+            /* Store any collections that were initialized and changed */
+            arr = apr_table_elts(g_msr->collections);
+            te = (apr_table_entry_t *)arr->elts;
+            for (n = 0; n < arr->nelts; n++) {
+                apr_table_t *col = (apr_table_t *)te[n].val;
+//                apr_table_t *orig_col = NULL;
+
+                if (g_msr->txcfg->debuglog_level >= 9) {
+                    msr_log(g_msr, 9, "Found loaded collection: %s", te[n].key);
+                }
+                /* Only store those collections that changed. */
+                if (apr_table_get(g_msr->collections_dirty, te[n].key)) {
+                    int x = collection_store(g_msr, col);
+
+                    if (g_msr->txcfg->debuglog_level >= 9) {
+                        msr_log(g_msr, 9, "Stored collection: %s (%d)", te[n].key, x);
+                    }
+                }
+#if 0
+                /* Re-populate the original values with the new ones. */
+                if ((orig_col = (apr_table_t *)apr_table_get(g_msr->collections_original, te[n].key)) != NULL) {
+                    const apr_array_header_t *orig_arr = apr_table_elts(orig_col);
+                    apr_table_entry_t *orig_te = (apr_table_entry_t *)orig_arr->elts;
+                    int m;
+
+                    for (m = 0; m < orig_arr->nelts; m++) {
+                        msc_string *mstr = (msc_string *)apr_table_get(col, orig_te[m].key);
+
+                        if (g_msr->txcfg->debuglog_level >= 9) {
+                            msr_log(g_msr, 9, "Updating original collection: %s.%s=%s", te[n].key, mstr->name, mstr->value);
+                        }
+                        //apr_table_setn(orig_col, orig_te[m].key, (void *)mstr );
+                        collection_original_setvar(g_msr, te[n].key, mstr);
+
+                        
+                    }
+                }
+#endif
+            }
+            apr_table_clear(g_msr->collections_dirty);
+            apr_table_clear(g_msr->collections_original);
         }
         else {
             fprintf(stderr, "Unknown type: \"%s\"\n", type);
