@@ -1,6 +1,6 @@
 /*
  * ModSecurity for Apache 2.x, http://www.modsecurity.org/
- * Copyright (c) 2004-2008 Breach Security, Inc. (http://www.breach.com/)
+ * Copyright (c) 2004-2009 Breach Security, Inc. (http://www.breach.com/)
  *
  * This product is released under the terms of the General Public Licence,
  * version 2 (GPLv2). Please refer to the file LICENSE (included with this
@@ -755,13 +755,15 @@ void msre_engine_destroy(msre_engine *engine) {
  * transaction phase.
  */
 #if defined(PERFORMANCE_MEASUREMENT)
+
+#define PERFORMANCE_MEASUREMENT_LOOP 1000
+
 static apr_status_t msre_ruleset_process_phase_(msre_ruleset *ruleset, modsec_rec *msr);
 
 apr_status_t msre_ruleset_process_phase(msre_ruleset *ruleset, modsec_rec *msr) {
     apr_array_header_t *arr = NULL;
     msre_rule **rules = NULL;
     apr_status_t rc;
-    apr_time_t time1;
     int i;
 
     switch (msr->phase) {
@@ -791,22 +793,24 @@ apr_status_t msre_ruleset_process_phase(msre_ruleset *ruleset, modsec_rec *msr)
         rule->execution_time = 0;
     }
 
-    time1 = apr_time_now();
-
-    for (i = 0; i < 10000; i++) {
+    for (i = 0; i < PERFORMANCE_MEASUREMENT_LOOP; i++) {
         rc = msre_ruleset_process_phase_(ruleset, msr);
     }
 
-    msr_log(msr, 1, "Phase %d: %" APR_TIME_T_FMT " usec", msr->phase, ((apr_time_now() - time1) / 10000));
+    msr_log(msr, 1, "Phase %d", msr->phase);
 
     rules = (msre_rule **)arr->elts;
     for (i = 0; i < arr->nelts; i++) {
         msre_rule *rule = rules[i];
+
+        /* Ignore markers, which are never processed. */
+        if (rule->placeholder == RULE_PH_MARKER) continue;
+
         msr_log(msr, 1, "Rule %pp [id \"%s\"][file \"%s\"][line \"%d\"]: %u usec", rule,
             ((rule->actionset != NULL)&&(rule->actionset->id != NULL)) ? rule->actionset->id : "-",
             rule->filename != NULL ? rule->filename : "-",
             rule->line_num,
-            (rule->execution_time / 10000));
+            (rule->execution_time / PERFORMANCE_MEASUREMENT_LOOP));
     }
 
     return rc;
@@ -1704,24 +1708,28 @@ static int execute_operator(msre_var *var, msre_rule *rule, modsec_rec *msr,
             var->value_len));
     }
 
-    #if !defined(PERFORMANCE_MEASUREMENT)
-    if (msr->txcfg->debuglog_level >= 4)
-    #endif
+    #if defined(PERFORMANCE_MEASUREMENT)
+    time_before_op = apr_time_now();
+    #else
+    if (msr->txcfg->debuglog_level >= 4) {
         time_before_op = apr_time_now();
-
+    }
+    #endif
+        
     rc = rule->op_metadata->execute(msr, rule, var, &my_error_msg);
 
-    #if !defined(PERFORMANCE_MEASUREMENT)
-    if (msr->txcfg->debuglog_level >= 4)
-    #endif
+    #if defined(PERFORMANCE_MEASUREMENT)
     {
+        /* Record performance but do not log anything. */
         apr_time_t t1 = apr_time_now();
-        #if defined(PERFORMANCE_MEASUREMENT)
         rule->op_time += (t1 - time_before_op);
-        #endif
-        msr_log(msr, 4, "Operator completed in %" APR_TIME_T_FMT " usec.",
-            (t1 - time_before_op));
     }
+    #else
+    if (msr->txcfg->debuglog_level >= 4) {
+        apr_time_t t1 = apr_time_now();
+        msr_log(msr, 4, "Operator completed in %" APR_TIME_T_FMT " usec.", (t1 - time_before_op));
+    }
+    #endif
 
     if (rc < 0) {
         msr_log(msr, 4, "Operator error: %s", my_error_msg);
@@ -1963,11 +1971,13 @@ static apr_status_t msre_rule_process_normal(msre_rule *rule, modsec_rec *msr) {
             }
         }
 
-        #if !defined(PERFORMANCE_MEASUREMENT)
-        if (msr->txcfg->debuglog_level >= 4)
-        #endif
+        #if defined(PERFORMANCE_MEASUREMENT)
+        time_before_trans = apr_time_now();
+        #else
+        if (msr->txcfg->debuglog_level >= 4) {
             time_before_trans = apr_time_now();
-
+        }
+        #endif
 
         /* Transform target. */
         {
@@ -2054,25 +2064,26 @@ static apr_status_t msre_rule_process_normal(msre_rule *rule, modsec_rec *msr) {
                         log_escape_nq_ex(mptmp, var->value, var->value_len), crec->hits);
                 }
 
-                #if !defined(PERFORMANCE_MEASUREMENT)
-                if (msr->txcfg->debuglog_level >= 4)
-                #endif
+                #if defined(PERFORMANCE_MEASUREMENT) 
                 {
                     apr_time_t t1 = apr_time_now();
-
-                    #if defined(PERFORMANCE_MEASUREMENT)
                     rule->trans_time += (t1 - time_before_trans);
-                    #endif
+                }
+                #else
+                if (msr->txcfg->debuglog_level >= 4) {
+                    apr_time_t t1 = apr_time_now();
 
                     msr_log(msr, 4, "Transformation completed in %" APR_TIME_T_FMT " usec.",
                         (t1 - time_before_trans));
                 }
+                #endif
 
                 rc = execute_operator(var, rule, msr, acting_actionset, mptmp);
 
                 if (rc < 0) {
                     return -1;
                 }
+
                 if (rc == RULE_MATCH) {
                     match_count++;
 
@@ -2084,6 +2095,7 @@ static apr_status_t msre_rule_process_normal(msre_rule *rule, modsec_rec *msr) {
                         return RULE_MATCH;
                     }
                 }
+
                 continue; /* next target */
             }
 
@@ -2135,19 +2147,19 @@ static apr_status_t msre_rule_process_normal(msre_rule *rule, modsec_rec *msr) {
                 if (multi_match && (k == 0 || tfnchanged)) {
                     invocations++;
 
-                    #if !defined(PERFORMANCE_MEASUREMENT)
-                    if (msr->txcfg->debuglog_level >= 4)
-                    #endif
+                    #if defined(PERFORMANCE_MEASUREMENT)
                     {
                         apr_time_t t1 = apr_time_now();
-
-                        #if defined(PERFORMANCE_MEASUREMENT)
                         rule->trans_time += (t1 - time_before_trans);
-                        #endif
+                    }
+                    #else
+                    if (msr->txcfg->debuglog_level >= 4) {
+                        apr_time_t t1 = apr_time_now();
 
                         msr_log(msr, 4, "Transformation completed in %" APR_TIME_T_FMT " usec.",
                             (t1 - time_before_trans));
                     }
+                    #endif
 
                     rc = execute_operator(var, rule, msr, acting_actionset, mptmp);
 
@@ -2269,19 +2281,19 @@ static apr_status_t msre_rule_process_normal(msre_rule *rule, modsec_rec *msr) {
         if (!multi_match || changed) {
             invocations++;
 
-            #if !defined(PERFORMANCE_MEASUREMENT)
-            if (msr->txcfg->debuglog_level >= 4)
-            #endif
+            #if defined(PERFORMANCE_MEASUREMENT)
             {
                 apr_time_t t1 = apr_time_now();
-
-                #if defined(PERFORMANCE_MEASUREMENT)
                 rule->trans_time += (t1 - time_before_trans);
-                #endif
+            }
+            #else
+            if (msr->txcfg->debuglog_level >= 4) {
+                apr_time_t t1 = apr_time_now();
 
                 msr_log(msr, 4, "Transformation completed in %" APR_TIME_T_FMT " usec.",
                     (t1 - time_before_trans));
             }
+            #endif
 
             rc = execute_operator(var, rule, msr, acting_actionset, mptmp);
 
