@@ -1,6 +1,6 @@
 /*
  * ModSecurity for Apache 2.x, http://www.modsecurity.org/
- * Copyright (c) 2004-2008 Breach Security, Inc. (http://www.breach.com/)
+ * Copyright (c) 2004-2009 Breach Security, Inc. (http://www.breach.com/)
  *
  * This product is released under the terms of the General Public Licence,
  * version 2 (GPLv2). Please refer to the file LICENSE (included with this
@@ -873,11 +873,25 @@ static int msre_op_validateDTD_execute(modsec_rec *msr, msre_rule *rule, msre_va
     xmlDtdPtr dtd;
 
     if ((msr->xml == NULL)||(msr->xml->doc == NULL)) {
-        *error_msg = apr_psprintf(msr->mp, "XML document tree could not be found for "
-            "DTD validation.");
+        *error_msg = apr_psprintf(msr->mp,
+            "XML document tree could not be found for DTD validation.");
         return -1;
     }
 
+    if (msr->xml->well_formed != 1) {
+        *error_msg = apr_psprintf(msr->mp,
+            "XML: DTD validation failed because content is not well formed.");
+        return 1;
+    }
+
+    /* Make sure there were no other generic processing errors */
+    if (msr->msc_reqbody_error) {
+        *error_msg = apr_psprintf(msr->mp,
+            "XML: DTD validation could not proceed due to previous"
+            " processing errors.");
+        return 1;
+    }
+
     dtd = xmlParseDTD(NULL, (const xmlChar *)rule->op_param); /* EHN support relative filenames */
     if (dtd == NULL) {
         *error_msg = apr_psprintf(msr->mp, "XML: Failed to load DTD: %s", rule->op_param);
@@ -891,6 +905,12 @@ static int msre_op_validateDTD_execute(modsec_rec *msr, msre_rule *rule, msre_va
         return -1;
     }
 
+    /* Send validator errors/warnings to msr_log */
+    /* NOTE: No xmlDtdSetValidErrors()? */
+    cvp->error = (xmlSchemaValidityErrorFunc)msr_log_error;
+    cvp->warning = (xmlSchemaValidityErrorFunc)msr_log_warn;
+    cvp->userData = msr;
+
     if (!xmlValidateDtd(cvp, msr->xml->doc, dtd)) {
         *error_msg = "XML: DTD validation failed.";
         xmlFreeValidCtxt(cvp);
@@ -923,11 +943,25 @@ static int msre_op_validateSchema_execute(modsec_rec *msr, msre_rule *rule, msre
     int rc;
 
     if ((msr->xml == NULL)||(msr->xml->doc == NULL)) {
-        *error_msg = apr_psprintf(msr->mp, "XML document tree could not be found for "
-            "Schema validation.");
+        *error_msg = apr_psprintf(msr->mp,
+            "XML document tree could not be found for schema validation.");
         return -1;
     }
 
+    if (msr->xml->well_formed != 1) {
+        *error_msg = apr_psprintf(msr->mp,
+            "XML: Schema validation failed because content is not well formed.");
+        return 1;
+    }
+
+    /* Make sure there were no other generic processing errors */
+    if (msr->msc_reqbody_error) {
+        *error_msg = apr_psprintf(msr->mp,
+            "XML: Schema validation could not proceed due to previous"
+            " processing errors.");
+        return 1;
+    }
+
     parserCtx = xmlSchemaNewParserCtxt(rule->op_param); /* ENH support relative filenames */
     if (parserCtx == NULL) {
         *error_msg = apr_psprintf(msr->mp, "XML: Failed to load Schema from file: %s",
@@ -935,6 +969,9 @@ static int msre_op_validateSchema_execute(modsec_rec *msr, msre_rule *rule, msre
         return -1;
     }
 
+    /* Send parser errors/warnings to msr_log */
+    xmlSchemaSetParserErrors(parserCtx, (xmlSchemaValidityErrorFunc)msr_log_error, (xmlSchemaValidityWarningFunc)msr_log_warn, msr);
+
     schema = xmlSchemaParse(parserCtx);
     if (schema == NULL) {
         *error_msg = apr_psprintf(msr->mp, "XML: Failed to load Schema: %s", rule->op_param);
@@ -950,6 +987,9 @@ static int msre_op_validateSchema_execute(modsec_rec *msr, msre_rule *rule, msre
         return -1;
     }
 
+    /* Send validator errors/warnings to msr_log */
+    xmlSchemaSetValidErrors(validCtx, (xmlSchemaValidityErrorFunc)msr_log_error, (xmlSchemaValidityWarningFunc)msr_log_warn, msr);
+
     rc = xmlSchemaValidateDoc(validCtx, msr->xml->doc);
     if (rc != 0) {
         *error_msg = "XML: Schema validation failed.";
